@@ -155,6 +155,14 @@ const char *pa_context_find_global_name(pa_context *c, uint32_t id)
 	return name;
 }
 
+static inline bool pa_endswith(const char *s, const char *sfx)
+{
+	size_t l1, l2;
+	l1 = strlen(s);
+	l2 = strlen(sfx);
+	return l1 >= l2 && pa_streq(s + l1 - l2, sfx);
+}
+
 struct global *pa_context_find_global_by_name(pa_context *c, uint32_t mask, const char *name)
 {
 	struct global *g;
@@ -165,9 +173,13 @@ struct global *pa_context_find_global_by_name(pa_context *c, uint32_t mask, cons
 		if ((g->mask & mask) == 0)
 			continue;
 		if (g->props != NULL &&
-		    (str = pw_properties_get(g->props, PW_KEY_NODE_NAME)) != NULL &&
-		    strcmp(str, name) == 0)
-			return g;
+		    (str = pw_properties_get(g->props, PW_KEY_NODE_NAME)) != NULL) {
+			if (strcmp(str, name) == 0)
+				return g;
+			if (pa_endswith(name, ".monitor") &&
+			    strncmp(str, name, strlen(name) - 8) == 0)
+				return g;
+		}
 		if (g->id == id || (g->id == (id & PA_IDX_MASK_DSP)))
 			return g;
 	}
@@ -369,8 +381,8 @@ static void device_event_info(void *object, const struct pw_device_info *info)
 				remove_params(&g->card_info.ports, id);
 				g->card_info.n_ports = 0;
 				break;
-			case SPA_PARAM_Route:
 			case SPA_PARAM_Profile:
+			case SPA_PARAM_Route:
 				break;
 			default:
 				do_enum = false;
@@ -381,9 +393,11 @@ static void device_event_info(void *object, const struct pw_device_info *info)
 			if (do_enum) {
 				switch (id) {
 				case SPA_PARAM_EnumProfile:
+				case SPA_PARAM_Profile:
 					g->card_info.pending_profiles = true;
 					break;
 				case SPA_PARAM_EnumRoute:
+				case SPA_PARAM_Route:
 					g->card_info.pending_ports = true;
 					break;
 				}
@@ -392,6 +406,8 @@ static void device_event_info(void *object, const struct pw_device_info *info)
 			}
 		}
 	}
+	if (i->driver == NULL)
+		i->driver = "PipeWire";
 	global_sync(g);
 }
 
@@ -501,8 +517,8 @@ static void device_event_param(void *object, int seq,
 			pw_log_warn("device %d: can't parse profile", g->id);
 			return;
 		}
-		g->card_info.active_profile = index;
 		pw_log_debug("device %d: current profile %d", g->id, index);
+		g->card_info.active_profile = index;
 		break;
 	}
 	case SPA_PARAM_EnumRoute:
@@ -540,17 +556,13 @@ static void device_event_param(void *object, int seq,
 			return;
 		}
 
-		if (direction == SPA_DIRECTION_OUTPUT)
-			g->card_info.active_port_output = index;
-		else
-			g->card_info.active_port_input = index;
-
 		pw_log_debug("device %d: active %s route %d", g->id,
 				direction == SPA_DIRECTION_OUTPUT ? "output" : "input",
 				index);
 
 		ng = find_node_for_route(c, g, device);
-		if (props && ng) {
+		if (props && ng && ng->node_info.active_port != index) {
+			ng->node_info.active_port = index;
 			parse_props(ng, props, true);
 			emit_event(c, ng, PA_SUBSCRIPTION_EVENT_CHANGE);
 		}
@@ -684,6 +696,7 @@ static void device_sync_ports(struct global *g)
 	n_ports = g->card_info.n_ports;
 	i->ports = calloc(n_ports+1, sizeof(pa_card_port_info *));
 	g->card_info.card_ports = calloc(n_ports, sizeof(pa_card_port_info));
+	g->card_info.port_devices = calloc(n_ports, sizeof(struct port_device));
 	i->n_ports = 0;
 
 	pw_log_debug("context %p: info for %d", g->context, g->id);
@@ -695,8 +708,9 @@ static void device_sync_ports(struct global *g)
 		enum spa_direction direction;
 		const char *name = NULL, *description = NULL;
 		enum spa_param_availability available = SPA_PARAM_AVAILABILITY_unknown;
-		struct spa_pod *profiles = NULL, *info = NULL;
+		struct spa_pod *profiles = NULL, *info = NULL, *devices = NULL;
 		pa_card_port_info *pi;
+		struct port_device *pd;
 
 		if (spa_pod_parse_object(p->param,
 				SPA_TYPE_OBJECT_ParamRoute, NULL,
@@ -707,6 +721,7 @@ static void device_sync_ports(struct global *g)
 				SPA_PARAM_ROUTE_priority,  SPA_POD_OPT_Int(&priority),
 				SPA_PARAM_ROUTE_available,  SPA_POD_OPT_Id(&available),
 				SPA_PARAM_ROUTE_info,  SPA_POD_OPT_Pod(&info),
+				SPA_PARAM_ROUTE_devices,  SPA_POD_OPT_Pod(&devices),
 				SPA_PARAM_ROUTE_profiles,  SPA_POD_OPT_Pod(&profiles)) < 0) {
 			pw_log_warn("device %d: can't parse route", g->id);
 			continue;
@@ -760,6 +775,11 @@ static void device_sync_ports(struct global *g)
 			pi->profiles = (pa_card_profile_info **)pi->profiles2;
 			break;
 		}
+		pd = &g->card_info.port_devices[j];
+		pd->n_devices = 0;
+		pd->devices = NULL;
+		if (devices)
+			pd->devices = spa_pod_get_array(devices, &pd->n_devices);
 		j++;
 	}
 	i->ports[j] = NULL;
@@ -773,6 +793,7 @@ static void device_sync(struct global *g)
 	if (g->card_info.pending_profiles) {
 		device_sync_profiles(g);
 		g->card_info.pending_profiles = false;
+		g->card_info.pending_ports = true;
 	}
 	if (g->card_info.pending_ports) {
 		device_sync_ports(g);
@@ -954,6 +975,10 @@ static void client_event_info(void *object, const struct pw_client_info *info)
 		i->driver = info->props ?
 			spa_dict_lookup(info->props, PW_KEY_PROTOCOL) : NULL;
 	}
+	if (i->name == NULL)
+		i->name = "Unknown";
+	if (i->driver == NULL)
+		i->name = "PipeWire";
 	global_sync(g);
 }
 
@@ -995,6 +1020,7 @@ static int metadata_property(void *object,
 	} else if (key && strcmp(key, METADATA_DEFAULT_SOURCE) == 0) {
 		val = value ? (uint32_t)atoi(value) : SPA_ID_INVALID;
 		changed = c->default_source != val;
+		c->default_source = val;
 	}
 	if (changed)
 		emit_event(global->context, global, PA_SUBSCRIPTION_EVENT_CHANGE);
@@ -1082,14 +1108,12 @@ static int set_mask(pa_context *c, struct global *g)
 		ginfo = &device_info;
 		spa_list_init(&g->card_info.profiles);
 		spa_list_init(&g->card_info.ports);
-		g->card_info.active_port_output = SPA_ID_INVALID;
-		g->card_info.active_port_input = SPA_ID_INVALID;
 	} else if (strcmp(g->type, PW_TYPE_INTERFACE_Node) == 0) {
 		if (g->props == NULL)
 			return 0;
 
-		if ((str = pw_properties_get(g->props, PW_KEY_PRIORITY_MASTER)) != NULL)
-			g->priority_master = pw_properties_parse_int(str);
+		if ((str = pw_properties_get(g->props, PW_KEY_PRIORITY_DRIVER)) != NULL)
+			g->priority_driver = pw_properties_parse_int(str);
 
 		if ((str = pw_properties_get(g->props, PW_KEY_MEDIA_CLASS)) == NULL) {
 			pw_log_debug("node %d without "PW_KEY_MEDIA_CLASS, g->id);
@@ -1135,6 +1159,7 @@ static int set_mask(pa_context *c, struct global *g)
 		g->node_info.mute = false;
 		g->node_info.base_volume = 1.0f;
 		g->node_info.volume_step = 1.0f / (PA_VOLUME_NORM+1);
+		g->node_info.active_port = SPA_ID_INVALID;
 	} else if (strcmp(g->type, PW_TYPE_INTERFACE_Port) == 0) {
 		if (g->props == NULL)
 			return 0;
@@ -1218,7 +1243,7 @@ static inline void insert_global(pa_context *c, struct global *global)
 	bool found = false;
 
 	spa_list_for_each(g, &c->globals, link) {
-		if (g->priority_master < global->priority_master) {
+		if (g->priority_driver < global->priority_driver) {
 			g = spa_list_prev(g, link);
 			found = true;
 			break;
@@ -1352,18 +1377,6 @@ static void on_success(pa_operation *o, void *userdata)
 	pa_operation_done(o);
 }
 
-void pa_context_ensure_registry(pa_context *c)
-{
-	if (c->registry != NULL)
-		return;
-
-	c->registry = pw_core_get_registry(c->core,
-			PW_VERSION_REGISTRY, 0);
-	pw_registry_add_listener(c->registry,
-			&c->registry_listener,
-			&registry_events, c);
-}
-
 SPA_EXPORT
 pa_operation* pa_context_subscribe(pa_context *c, pa_subscription_mask_t m, pa_context_success_cb_t cb, void *userdata)
 {
@@ -1377,8 +1390,6 @@ pa_operation* pa_context_subscribe(pa_context *c, pa_subscription_mask_t m, pa_c
 
 	c->subscribe_mask = m;
 
-	pa_context_ensure_registry(c);
-
 	o = pa_operation_new(c, NULL, on_success, sizeof(struct success_data));
 	d = o->userdata;
 	d->cb = cb;
@@ -1406,7 +1417,11 @@ pa_context *pa_context_new_with_proplist(pa_mainloop_api *mainloop, const char *
 		pw_properties_update_proplist(props, p);
 
 	loop = mainloop->userdata;
-	context = pw_context_new(loop, NULL, sizeof(struct pa_context));
+	context = pw_context_new(loop,
+			pw_properties_new(
+				PW_KEY_CONTEXT_PROFILE_MODULES, "default,rtkit",
+				NULL),
+			sizeof(struct pa_context));
 	if (context == NULL)
 		return NULL;
 
@@ -1560,6 +1575,12 @@ int pa_context_connect(pa_context *c, const char *server, pa_context_flags_t fla
 	}
 	pw_core_add_listener(c->core, &c->core_listener, &core_events, c);
 
+	c->registry = pw_core_get_registry(c->core,
+			PW_VERSION_REGISTRY, 0);
+	pw_registry_add_listener(c->registry,
+			&c->registry_listener,
+			&registry_events, c);
+
 exit:
 	pa_context_unref(c);
 
@@ -1676,8 +1697,6 @@ pa_operation* pa_context_set_default_sink(pa_context *c, const char *name, pa_co
 	pa_operation *o;
 	struct default_node *d;
 
-	pa_context_ensure_registry(c);
-
 	o = pa_operation_new(c, NULL, do_default_node, sizeof(*d));
 	d = o->userdata;
 	d->mask = PA_SUBSCRIPTION_MASK_SINK;
@@ -1696,8 +1715,6 @@ pa_operation* pa_context_set_default_source(pa_context *c, const char *name, pa_
 	pa_operation *o;
 	struct default_node *d;
 
-	pa_context_ensure_registry(c);
-
 	o = pa_operation_new(c, NULL, do_default_node, sizeof(*d));
 	d = o->userdata;
 	d->mask = PA_SUBSCRIPTION_MASK_SOURCE;
