@@ -112,8 +112,6 @@ static void stream_state_changed(void *data, enum pw_stream_state old,
 		if (!s->disconnecting) {
 			pa_context_set_error(c, PA_ERR_KILLED);
 			pa_stream_set_state(s, PA_STREAM_FAILED);
-		} else {
-			pa_stream_set_state(s, PA_STREAM_TERMINATED);
 		}
 		break;
 	case PW_STREAM_STATE_CONNECTING:
@@ -619,7 +617,8 @@ static void stream_unlink(pa_stream *s)
 	}
 
 	spa_list_remove(&s->link);
-	pw_stream_set_active(s->stream, false);
+	if (s->stream)
+		pw_stream_set_active(s->stream, false);
 
 	s->context = NULL;
 	pa_stream_unref(s);
@@ -753,11 +752,9 @@ const char *pa_stream_get_device_name(PA_CONST pa_stream *s)
 
 	PA_CHECK_VALIDITY_RETURN_NULL(s->context, s->state == PA_STREAM_READY, PA_ERR_BADSTATE);
 	PA_CHECK_VALIDITY_RETURN_NULL(s->context, s->direction != PA_STREAM_UPLOAD, PA_ERR_BADSTATE);
-//	PA_CHECK_VALIDITY_RETURN_NULL(s->context, s->device_name, PA_ERR_BADSTATE);
-
-	if (s->device_name == NULL)
-		return "unnamed";
+	PA_CHECK_VALIDITY_RETURN_NULL(s->context, s->device_name, PA_ERR_BADSTATE);
 
+	pw_log_trace("stream %p: %s %d", s, s->device_name, s->device_index);
 	return s->device_name;
 }
 
@@ -843,7 +840,7 @@ static int create_stream(pa_stream_direction_t direction,
 	PA_CHECK_VALIDITY(s->context, (flags & (PA_STREAM_ADJUST_LATENCY|PA_STREAM_EARLY_REQUESTS)) != (PA_STREAM_ADJUST_LATENCY|PA_STREAM_EARLY_REQUESTS), PA_ERR_INVALID);
 
 
-	pw_log_debug("stream %p: connect %s %08x", s, dev, flags);
+	pw_log_info("stream %p: connect %s %08x", s, dev, flags);
 
 	name = pa_proplist_gets(s->proplist, PA_PROP_MEDIA_NAME);
 
@@ -1020,6 +1017,7 @@ int pa_stream_connect_record(
 static void on_disconnected(pa_operation *o, void *userdata)
 {
 	pa_stream *s = o->stream;
+	pw_log_debug("stream %p", s);
 	pa_stream_set_state(s, PA_STREAM_TERMINATED);
 }
 
@@ -1561,7 +1559,8 @@ pa_operation* pa_stream_flush(pa_stream *s, pa_stream_success_cb_t cb, void *use
 		spa_list_remove(&m->link);
 		spa_list_append(&s->free, &m->link);
 		m->user_data = NULL;
-		b->user_data = NULL;
+		if (b)
+			b->user_data = NULL;
 	}
 	s->ready_bytes = 0;
 	s->timing_info.write_index = s->timing_info.read_index = 0;
