@@ -339,6 +339,50 @@ static int impl_sync(void *object, int seq)
 	return 0;
 }
 
+static struct spa_pod *build_profile(struct impl *this, struct spa_pod_builder *b,
+		uint32_t id, uint32_t index)
+{
+	struct spa_pod_frame f[2];
+	const char *name, *desc;
+
+	switch (index) {
+	case 0:
+		name = "off";
+		desc = "Off";
+		break;
+	case 1:
+		name = "on";
+		desc = "On";
+		break;
+	default:
+		errno = -EINVAL;
+		return NULL;
+	}
+
+	spa_pod_builder_push_object(b, &f[0], SPA_TYPE_OBJECT_ParamProfile, id);
+	spa_pod_builder_add(b,
+		SPA_PARAM_PROFILE_index,   SPA_POD_Int(index),
+		SPA_PARAM_PROFILE_name, SPA_POD_String(name),
+		SPA_PARAM_PROFILE_description, SPA_POD_String(desc),
+		0);
+	if (index == 1) {
+		spa_pod_builder_prop(b, SPA_PARAM_PROFILE_classes, 0);
+		spa_pod_builder_push_struct(b, &f[1]);
+		if (this->n_capture) {
+			spa_pod_builder_add_struct(b,
+				SPA_POD_String("Audio/Source"),
+				SPA_POD_Int(this->n_capture));
+		}
+		if (this->n_playback) {
+			spa_pod_builder_add_struct(b,
+				SPA_POD_String("Audio/Sink"),
+				SPA_POD_Int(this->n_playback));
+		}
+		spa_pod_builder_pop(b, &f[1]);
+	}
+	return spa_pod_builder_pop(b, &f[0]);
+
+}
 static int impl_enum_params(void *object, int seq,
 			    uint32_t id, uint32_t start, uint32_t num,
 			    const struct spa_pod *filter)
@@ -363,34 +407,10 @@ static int impl_enum_params(void *object, int seq,
 	switch (id) {
 	case SPA_PARAM_EnumProfile:
 	{
-		struct spa_pod_frame f[2];
 		switch (result.index) {
 		case 0:
-			param = spa_pod_builder_add_object(&b,
-				SPA_TYPE_OBJECT_ParamProfile, id,
-				SPA_PARAM_PROFILE_index,   SPA_POD_Int(0),
-				SPA_PARAM_PROFILE_name, SPA_POD_String("Off"));
-			break;
 		case 1:
-			spa_pod_builder_push_object(&b, &f[0], SPA_TYPE_OBJECT_ParamProfile, id);
-			spa_pod_builder_add(&b,
-				SPA_PARAM_PROFILE_index,   SPA_POD_Int(1),
-				SPA_PARAM_PROFILE_name, SPA_POD_String("On"),
-				0);
-			spa_pod_builder_prop(&b, SPA_PARAM_PROFILE_classes, 0);
-			spa_pod_builder_push_struct(&b, &f[1]);
-			if (this->n_capture) {
-				spa_pod_builder_add_struct(&b,
-					SPA_POD_String("Audio/Source"),
-					SPA_POD_Int(this->n_capture));
-			}
-			if (this->n_playback) {
-				spa_pod_builder_add_struct(&b,
-					SPA_POD_String("Audio/Sink"),
-					SPA_POD_Int(this->n_playback));
-			}
-			spa_pod_builder_pop(&b, &f[1]);
-			param = spa_pod_builder_pop(&b, &f[0]);
+			param = build_profile(this, &b, id, result.index);
 			break;
 		default:
 			return 0;
@@ -401,9 +421,7 @@ static int impl_enum_params(void *object, int seq,
 	{
 		switch (result.index) {
 		case 0:
-			param = spa_pod_builder_add_object(&b,
-				SPA_TYPE_OBJECT_ParamProfile, id,
-				SPA_PARAM_PROFILE_index, SPA_POD_Int(this->profile));
+			param = build_profile(this, &b, id, this->profile);
 			break;
 		default:
 			return 0;
