@@ -337,17 +337,6 @@ static int impl_node_remove_port(void *object, enum spa_direction direction, uin
 	return -ENOTSUP;
 }
 
-static void recycle_buffer(struct state *this, uint32_t buffer_id)
-{
-	struct buffer *b = &this->buffers[buffer_id];
-
-	if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_OUT)) {
-		spa_log_trace_fp(this->log, NAME " %p: recycle buffer %u", this, buffer_id);
-		spa_list_append(&this->free, &b->link);
-		SPA_FLAG_CLEAR(b->flags, BUFFER_FLAG_OUT);
-	}
-}
-
 static int
 impl_node_port_enum_params(void *object, int seq,
 			   enum spa_direction direction, uint32_t port_id,
@@ -395,7 +384,7 @@ impl_node_port_enum_params(void *object, int seq,
 		param = spa_pod_builder_add_object(&b,
 			SPA_TYPE_OBJECT_ParamBuffers, id,
 			SPA_PARAM_BUFFERS_buffers, SPA_POD_CHOICE_RANGE_Int(2, 1, MAX_BUFFERS),
-			SPA_PARAM_BUFFERS_blocks,  SPA_POD_Int(1),
+			SPA_PARAM_BUFFERS_blocks,  SPA_POD_Int(this->blocks),
 			SPA_PARAM_BUFFERS_size,    SPA_POD_CHOICE_RANGE_Int(
 							this->props.max_latency * this->frame_size,
 							this->props.min_latency * this->frame_size,
@@ -623,7 +612,7 @@ static int impl_node_port_reuse_buffer(void *object, uint32_t port_id, uint32_t
 	if (buffer_id >= this->n_buffers)
 		return -EINVAL;
 
-	recycle_buffer(this, buffer_id);
+	spa_alsa_recycle_buffer(this, buffer_id);
 
 	return 0;
 }
@@ -643,7 +632,7 @@ static int impl_node_process(void *object)
 		return SPA_STATUS_HAVE_DATA;
 
 	if (io->buffer_id < this->n_buffers) {
-		recycle_buffer(this, io->buffer_id);
+		spa_alsa_recycle_buffer(this, io->buffer_id);
 		io->buffer_id = SPA_ID_INVALID;
 	}
 
@@ -655,6 +644,7 @@ static int impl_node_process(void *object)
 
 	b = spa_list_first(&this->ready, struct buffer, link);
 	spa_list_remove(&b->link);
+	SPA_FLAG_SET(b->flags, BUFFER_FLAG_OUT);
 
 	spa_log_trace_fp(this->log, NAME " %p: dequeue buffer %d", this, b->id);
 
@@ -780,6 +770,8 @@ impl_init(const struct spa_handle_factory *factory,
 	for (i = 0; info && i < info->n_items; i++) {
 		if (!strcmp(info->items[i].key, SPA_KEY_API_ALSA_PATH)) {
 			snprintf(this->props.device, 63, "%s", info->items[i].value);
+		} else if (!strcmp(info->items[i].key, SPA_KEY_AUDIO_CHANNELS)) {
+			this->default_channels = atoi(info->items[i].value);
 		}
 	}
 	return 0;
