@@ -101,12 +101,15 @@ struct state {
 	bool have_format;
 	struct spa_audio_info current_format;
 
+	unsigned int default_channels;
+
 	snd_pcm_uframes_t buffer_frames;
 	snd_pcm_uframes_t period_frames;
 	snd_pcm_format_t format;
 	int rate;
 	int channels;
 	size_t frame_size;
+	int blocks;
 	int rate_denom;
 	uint32_t delay;
 	uint32_t read_size;
@@ -141,6 +144,7 @@ struct state {
 	unsigned int alsa_recovering:1;
 	unsigned int following:1;
 	unsigned int matching:1;
+	unsigned int resample:1;
 
 	int64_t sample_count;
 
@@ -172,6 +176,8 @@ int spa_alsa_close(struct state *state);
 int spa_alsa_write(struct state *state, snd_pcm_uframes_t silence);
 int spa_alsa_read(struct state *state, snd_pcm_uframes_t silence);
 
+void spa_alsa_recycle_buffer(struct state *state, uint32_t buffer_id);
+
 #ifdef __cplusplus
 } /* extern "C" */
 #endif
