@@ -80,10 +80,9 @@ struct impl {
 	struct spa_callbacks callbacks;
 
 	unsigned int add_listener:1;
-	unsigned int use_converter:1;
 	unsigned int have_format:1;
 	unsigned int started:1;
-	unsigned int master:1;
+	unsigned int driver:1;
 };
 
 /** \endcond */
@@ -146,7 +145,7 @@ static int link_io(struct impl *this)
 {
 	int res;
 
-	if (!this->use_converter)
+	if (this->convert == NULL)
 		return 0;
 
 	spa_log_debug(this->log, NAME " %p: controls", this);
@@ -213,6 +212,8 @@ static int debug_params(struct impl *this, struct spa_node *node,
         spa_log_error(this->log, "params %s: %d:%d (%s) %s",
 			spa_debug_type_find_name(spa_type_param, id),
 			direction, port_id, debug, spa_strerror(err));
+	if (err == -EBUSY)
+		return 0;
 
         state = 0;
         while (true) {
@@ -249,7 +250,7 @@ static int negotiate_buffers(struct impl *this)
 	struct spa_data *datas;
 	uint32_t follower_flags, conv_flags;
 
-	spa_log_debug(this->log, "%p: %d", this, this->n_buffers);
+	spa_log_debug(this->log, NAME" %p: %d", this, this->n_buffers);
 
 	if (this->n_buffers > 0)
 		return 0;
@@ -260,9 +261,13 @@ static int negotiate_buffers(struct impl *this)
 				this->direction, 0,
 				SPA_PARAM_Buffers, &state,
 				param, &param, &b)) < 0) {
-		debug_params(this, this->follower, this->direction, 0,
+		if (res == -ENOENT)
+			param = NULL;
+		else {
+			debug_params(this, this->follower, this->direction, 0,
 				SPA_PARAM_Buffers, param, "follower buffers", res);
-		return -ENOTSUP;
+			return res;
+		}
 	}
 
 	state = 0;
@@ -275,6 +280,8 @@ static int negotiate_buffers(struct impl *this)
 				SPA_PARAM_Buffers, param, "convert buffers", res);
 		return -ENOTSUP;
 	}
+	if (param == NULL)
+		return -ENOTSUP;
 
 	spa_pod_fixate(param);
 
@@ -343,7 +350,7 @@ static int configure_format(struct impl *this, uint32_t flags, const struct spa_
 	if (format && spa_log_level_enabled(this->log, SPA_LOG_LEVEL_DEBUG))
 		spa_debug_format(0, NULL, format);
 
-	if (this->use_converter) {
+	if (this->convert) {
 		if ((res = spa_node_port_set_param(this->convert,
 					   SPA_DIRECTION_REVERSE(this->direction), 0,
 					   SPA_PARAM_Format, flags,
@@ -445,7 +452,7 @@ static int negotiate_format(struct impl *this)
 
 	spa_pod_builder_init(&b, buffer, sizeof(buffer));
 
-	spa_log_debug(this->log, NAME "%p: negiotiate", this);
+	spa_log_debug(this->log, NAME " %p: negiotiate", this);
 
 	state = 0;
 	format = NULL;
@@ -457,12 +464,16 @@ static int negotiate_format(struct impl *this)
 				this->direction, 0,
 				SPA_PARAM_EnumFormat, &state,
 				format, &format, &b)) < 0) {
-		debug_params(this, this->follower, this->direction, 0,
-				SPA_PARAM_EnumFormat, format, "follower format", res);
-		return -ENOTSUP;
+		if (res == -ENOENT)
+			format = NULL;
+		else {
+			debug_params(this, this->follower, this->direction, 0,
+					SPA_PARAM_EnumFormat, format, "follower format", res);
+			return res;
+		}
 	}
 
-	if (this->use_converter) {
+	if (this->convert) {
 		state = 0;
 		if ((res = spa_node_port_enum_params_sync(this->convert,
 					SPA_DIRECTION_REVERSE(this->direction), 0,
@@ -474,6 +485,8 @@ static int negotiate_format(struct impl *this)
 			return -ENOTSUP;
 		}
 	}
+	if (format == NULL)
+		return -ENOTSUP;
 
 	spa_pod_fixate(format);
 
@@ -498,7 +511,6 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 			return res;
 		if ((res = negotiate_buffers(this)) < 0)
 			return res;
-		this->started = true;
 		break;
 	case SPA_NODE_COMMAND_Suspend:
 		configure_format(this, 0, NULL);
@@ -523,6 +535,11 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 			return res;
 		}
 	}
+	switch (SPA_NODE_COMMAND_ID(command)) {
+	case SPA_NODE_COMMAND_Start:
+		this->started = true;
+		break;
+	}
 	return res;
 }
 
@@ -531,20 +548,26 @@ static void convert_node_info(void *data, const struct spa_node_info *info)
 	struct impl *this = data;
 	uint32_t i;
 
-	for (i = 0; i < info->n_params; i++) {
-		uint32_t idx = SPA_ID_INVALID;
+	if (info->change_mask & SPA_NODE_CHANGE_MASK_FLAGS) {
+		this->info.change_mask |= SPA_NODE_CHANGE_MASK_FLAGS;
+		this->info.flags = info->flags;
+	}
+	if (info->change_mask & SPA_NODE_CHANGE_MASK_PARAMS) {
+		for (i = 0; i < info->n_params; i++) {
+			uint32_t idx = SPA_ID_INVALID;
 
-		switch (info->params[i].id) {
-		case SPA_PARAM_PropInfo:
-			idx = 1;
-			break;
-		case SPA_PARAM_Props:
-			idx = 2;
-			break;
-		}
-		if (idx != SPA_ID_INVALID) {
-			this->params[idx] = info->params[i];
-			this->info.change_mask |= SPA_NODE_CHANGE_MASK_PARAMS;
+			switch (info->params[i].id) {
+			case SPA_PARAM_PropInfo:
+				idx = 1;
+				break;
+			case SPA_PARAM_Props:
+				idx = 2;
+				break;
+			}
+			if (idx != SPA_ID_INVALID) {
+				this->params[idx] = info->params[i];
+				this->info.change_mask |= SPA_NODE_CHANGE_MASK_PARAMS;
+			}
 		}
 	}
 	emit_node_info(this, false);
@@ -642,7 +665,7 @@ static int follower_ready(void *data, int status)
 
 	spa_log_trace_fp(this->log, NAME " %p: ready %d", this, status);
 
-	this->master = true;
+	this->driver = true;
 
 	if (this->direction == SPA_DIRECTION_OUTPUT)
 		status = spa_node_process(this->convert);
@@ -655,7 +678,7 @@ static int follower_reuse_buffer(void *data, uint32_t port_id, uint32_t buffer_i
 	int res;
 	struct impl *this = data;
 
-	if (this->use_converter)
+	if (this->convert)
 		res = spa_node_port_reuse_buffer(this->convert, port_id, buffer_id);
 	else
 		res = spa_node_call_reuse_buffer(&this->callbacks, port_id, buffer_id);
@@ -698,7 +721,7 @@ static int impl_node_add_listener(void *object,
 		spa_node_add_listener(this->follower, &l, &follower_node_events, this);
 		spa_hook_remove(&l);
 
-		if (this->use_converter) {
+		if (this->convert) {
 			spa_zero(l);
 			spa_node_add_listener(this->convert, &l, &convert_node_events, this);
 			spa_hook_remove(&l);
@@ -865,35 +888,35 @@ impl_node_port_reuse_buffer(void *object, uint32_t port_id, uint32_t buffer_id)
 static int impl_node_process(void *object)
 {
 	struct impl *this = object;
-	int status;
+	int status = 0;
 
-	spa_log_trace_fp(this->log, "%p: process convert:%u master:%d",
-			this, this->use_converter, this->master);
+	spa_log_trace_fp(this->log, "%p: process convert:%p driver:%d",
+			this, this->convert, this->driver);
 
 	if (this->direction == SPA_DIRECTION_INPUT) {
-		if (this->use_converter)
+		if (this->convert)
 			status = spa_node_process(this->convert);
 	}
 
-	status = spa_node_process(this->follower);
+	if (status >= 0)
+		status = spa_node_process(this->follower);
 
 	if (this->direction == SPA_DIRECTION_OUTPUT &&
-	    !this->master && this->use_converter) {
-		while (true) {
+	    !this->driver && this->convert) {
+		while (status > 0) {
 			status = spa_node_process(this->convert);
-			if (status & SPA_STATUS_HAVE_DATA)
+			if (status & (SPA_STATUS_HAVE_DATA | SPA_STATUS_DRAINED))
 				break;
-
 			if (status & SPA_STATUS_NEED_DATA) {
 				status = spa_node_process(this->follower);
-				if (!(status & SPA_STATUS_HAVE_DATA))
+				if (status & SPA_STATUS_NEED_DATA)
 					break;
 			}
 		}
 	}
 	spa_log_trace_fp(this->log, "%p: process status:%d", this, status);
 
-	this->master = false;
+	this->driver = false;
 
 	return status;
 }
@@ -945,6 +968,8 @@ static int impl_clear(struct spa_handle *handle)
 	spa_hook_remove(&this->follower_listener);
 	spa_node_set_callbacks(this->follower, NULL, NULL);
 
+	spa_handle_clear(this->hnd_convert);
+
 	if (this->buffers)
 		free(this->buffers);
 	this->buffers = NULL;
@@ -1033,8 +1058,13 @@ impl_init(const struct spa_handle_factory *factory,
 	this->convert = iface;
 	this->target = this->convert;
 
-	this->info_all = SPA_NODE_CHANGE_MASK_PARAMS;
+	this->info_all = SPA_NODE_CHANGE_MASK_FLAGS |
+		SPA_NODE_CHANGE_MASK_PARAMS;
 	this->info = SPA_NODE_INFO_INIT();
+	this->info.flags = SPA_NODE_FLAG_RT |
+		SPA_NODE_FLAG_IN_PORT_CONFIG |
+		SPA_NODE_FLAG_OUT_PORT_CONFIG |
+		SPA_NODE_FLAG_NEED_CONFIGURE;
 	this->params[0] = SPA_PARAM_INFO(SPA_PARAM_EnumFormat, SPA_PARAM_INFO_READ);
 	this->params[1] = SPA_PARAM_INFO(SPA_PARAM_PropInfo, SPA_PARAM_INFO_READ);
 	this->params[2] = SPA_PARAM_INFO(SPA_PARAM_Props, SPA_PARAM_INFO_READWRITE);
@@ -1050,7 +1080,6 @@ impl_init(const struct spa_handle_factory *factory,
 
 	spa_node_add_listener(this->convert,
 			&this->convert_listener, &convert_node_events, this);
-	this->use_converter = true;
 
 	configure_adapt(this);
 
