@@ -31,6 +31,7 @@
 #include <spa/support/cpu.h>
 #include <spa/utils/list.h>
 #include <spa/utils/names.h>
+#include <spa/node/keys.h>
 #include <spa/node/node.h>
 #include <spa/node/io.h>
 #include <spa/node/utils.h>
@@ -46,9 +47,11 @@
 #define DEFAULT_RATE		48000
 #define DEFAULT_CHANNELS	2
 
-#define MAX_SAMPLES	8192
+#define DEFAULT_SAMPLES	8192
 #define MAX_BUFFERS	32
-#define MAX_DATAS	32
+#define MAX_DATAS	64
+
+#define DEFAULT_CONTROL_BUFFER_SIZE	32768
 
 struct impl;
 
@@ -102,6 +105,9 @@ struct port {
 	uint32_t n_buffers;
 
 	struct spa_list queue;
+
+	struct spa_pod_sequence *ctrl;
+	uint32_t ctrl_offset;
 };
 
 struct impl {
@@ -119,6 +125,7 @@ struct impl {
 	struct spa_param_info params[8];
 
 
+	struct port control_port;
 	struct port in_port;
 	struct port out_port;
 
@@ -128,10 +135,14 @@ struct impl {
 	uint32_t cpu_flags;
 };
 
-#define CHECK_PORT(this,d,id)		(id == 0)
+#define IS_CONTROL_PORT(this,d,id)	(id == 1 && d == SPA_DIRECTION_INPUT)
+#define IS_DATA_PORT(this,d,id)		(id == 0)
+
+#define CHECK_PORT(this,d,id)		(IS_CONTROL_PORT(this,d,id) || IS_DATA_PORT(this,d,id))
+#define GET_CONTROL_PORT(this,id)	(&this->control_port)
 #define GET_IN_PORT(this,id)		(&this->in_port)
 #define GET_OUT_PORT(this,id)		(&this->out_port)
-#define GET_PORT(this,d,id)		(d == SPA_DIRECTION_INPUT ? GET_IN_PORT(this,id) : GET_OUT_PORT(this,id))
+#define GET_PORT(this,d,id)		(IS_CONTROL_PORT(this,d,id) ? GET_CONTROL_PORT(this,id) : (d == SPA_DIRECTION_INPUT ? GET_IN_PORT(this,id) : GET_OUT_PORT(this,id)))
 
 #define _MASK(ch)	(1ULL << SPA_AUDIO_CHANNEL_ ## ch)
 #define STEREO	(_MASK(FL)|_MASK(FR))
@@ -375,6 +386,25 @@ static int apply_props(struct impl *this, const struct spa_pod *param)
 	return changed;
 }
 
+static int apply_midi(struct impl *this, const struct spa_pod *value)
+{
+	const uint8_t *val = SPA_POD_BODY(value);
+	uint32_t size = SPA_POD_BODY_SIZE(value);
+	struct props *p = &this->props;
+
+	if (size < 3)
+		return -EINVAL;
+
+	if ((val[0] & 0xf0) != 0xb0 || val[1] != 7)
+		return 0;
+
+	p->volume = val[2] / 127.0;
+	if (this->mix.set_volume)
+		channelmix_set_volume(&this->mix, p->volume, p->mute,
+			p->n_channel_volumes, p->channel_volumes);
+	return 1;
+}
+
 static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 {
 	return -ENOTSUP;
@@ -437,6 +467,8 @@ impl_node_add_listener(void *object,
 {
 	struct impl *this = object;
 	struct spa_hook_list save;
+	struct spa_dict_item items[2];
+	uint32_t n_items = 0;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
@@ -446,6 +478,12 @@ impl_node_add_listener(void *object,
 	emit_port_info(this, GET_IN_PORT(this, 0), true);
 	emit_port_info(this, GET_OUT_PORT(this, 0), true);
 
+	struct port *control_port = GET_CONTROL_PORT(this, 1);
+	items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_PORT_NAME, "control");
+	items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_FORMAT_DSP, "8 bit raw midi");
+	control_port->info.props = &SPA_DICT_INIT(items, n_items);
+	emit_port_info(this, control_port, true);
+
 	spa_hook_list_join(&this->hooks, &save);
 
 	return 0;
@@ -479,28 +517,40 @@ static int port_enum_formats(void *object,
 			     struct spa_pod_builder *builder)
 {
 	struct impl *this = object;
-	struct port *other;
-
-	other = GET_PORT(this, SPA_DIRECTION_REVERSE(direction), 0);
 
 	switch (index) {
 	case 0:
-		if (other->have_format) {
+		if (IS_CONTROL_PORT(this, direction, port_id)) {
 			*param = spa_pod_builder_add_object(builder,
 				SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat,
-				SPA_FORMAT_mediaType,      SPA_POD_Id(SPA_MEDIA_TYPE_audio),
-				SPA_FORMAT_mediaSubtype,   SPA_POD_Id(SPA_MEDIA_SUBTYPE_raw),
-				SPA_FORMAT_AUDIO_format,   SPA_POD_Id(SPA_AUDIO_FORMAT_F32P),
-				SPA_FORMAT_AUDIO_rate,     SPA_POD_Int(other->format.info.raw.rate),
-				SPA_FORMAT_AUDIO_channels, SPA_POD_CHOICE_RANGE_Int(DEFAULT_CHANNELS, 1, INT32_MAX));
+				SPA_FORMAT_mediaType,	   SPA_POD_Id(SPA_MEDIA_TYPE_application),
+				SPA_FORMAT_mediaSubtype,   SPA_POD_Id(SPA_MEDIA_SUBTYPE_control));
 		} else {
-			*param = spa_pod_builder_add_object(builder,
-				SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat,
+			struct spa_pod_frame f;
+			struct port *other;
+
+			other = GET_PORT(this, SPA_DIRECTION_REVERSE(direction), 0);
+
+			spa_pod_builder_push_object(builder, &f,
+				SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat);
+			spa_pod_builder_add(builder,
 				SPA_FORMAT_mediaType,      SPA_POD_Id(SPA_MEDIA_TYPE_audio),
 				SPA_FORMAT_mediaSubtype,   SPA_POD_Id(SPA_MEDIA_SUBTYPE_raw),
 				SPA_FORMAT_AUDIO_format,   SPA_POD_Id(SPA_AUDIO_FORMAT_F32P),
-				SPA_FORMAT_AUDIO_rate,     SPA_POD_CHOICE_RANGE_Int(DEFAULT_RATE, 1, INT32_MAX),
-				SPA_FORMAT_AUDIO_channels, SPA_POD_CHOICE_RANGE_Int(DEFAULT_CHANNELS, 1, INT32_MAX));
+				0);
+			if (other->have_format) {
+				spa_pod_builder_add(builder,
+					SPA_FORMAT_AUDIO_rate, SPA_POD_Int(other->format.info.raw.rate),
+					0);
+			} else {
+				spa_pod_builder_add(builder,
+					SPA_FORMAT_AUDIO_rate, SPA_POD_CHOICE_RANGE_Int(DEFAULT_RATE, 1, INT32_MAX),
+					0);
+			}
+			spa_pod_builder_add(builder,
+				SPA_FORMAT_AUDIO_channels, SPA_POD_CHOICE_RANGE_Int(DEFAULT_CHANNELS, 1, INT32_MAX),
+				0);
+			*param = spa_pod_builder_pop(builder, &f);
 		}
 		break;
 	default:
@@ -554,7 +604,13 @@ impl_node_port_enum_params(void *object, int seq,
 		if (result.index > 0)
 			return 0;
 
-		param = spa_format_audio_raw_build(&b, id, &port->format.info.raw);
+		if (IS_CONTROL_PORT(this, direction, port_id))
+			param = spa_pod_builder_add_object(&b,
+				SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat,
+				SPA_FORMAT_mediaType,	   SPA_POD_Id(SPA_MEDIA_TYPE_application),
+				SPA_FORMAT_mediaSubtype,   SPA_POD_Id(SPA_MEDIA_SUBTYPE_control));
+		else
+			param = spa_format_audio_raw_build(&b, id, &port->format.info.raw);
 		break;
 
 	case SPA_PARAM_Buffers:
@@ -566,29 +622,45 @@ impl_node_port_enum_params(void *object, int seq,
 		if (result.index > 0)
 			return 0;
 
-		if (other->n_buffers > 0) {
-			buffers = other->n_buffers;
-			size = other->size / other->stride;
+		if (IS_CONTROL_PORT(this, direction, port_id)) {
+			param = spa_pod_builder_add_object(&b,
+				SPA_TYPE_OBJECT_ParamBuffers, id,
+				SPA_PARAM_BUFFERS_buffers, SPA_POD_CHOICE_RANGE_Int(1, 1, MAX_BUFFERS),
+				SPA_PARAM_BUFFERS_blocks,  SPA_POD_Int(1),
+				SPA_PARAM_BUFFERS_size,    SPA_POD_CHOICE_RANGE_Int(
+								DEFAULT_CONTROL_BUFFER_SIZE,
+								1024,
+								INT32_MAX),
+				SPA_PARAM_BUFFERS_stride,  SPA_POD_Int(1),
+				SPA_PARAM_BUFFERS_align,   SPA_POD_Int(16));
 		} else {
-			buffers = 1;
-			size = MAX_SAMPLES;
-		}
+			if (other->n_buffers > 0) {
+				buffers = other->n_buffers;
+				size = other->size / other->stride;
+			} else {
+				buffers = 1;
+				size = DEFAULT_SAMPLES;
+			}
 
-		param = spa_pod_builder_add_object(&b,
-			SPA_TYPE_OBJECT_ParamBuffers, id,
-			SPA_PARAM_BUFFERS_buffers, SPA_POD_CHOICE_RANGE_Int(buffers, 1, MAX_BUFFERS),
-			SPA_PARAM_BUFFERS_blocks,  SPA_POD_Int(port->blocks),
-			SPA_PARAM_BUFFERS_size,    SPA_POD_CHOICE_RANGE_Int(
-							size * port->stride,
-							16 * port->stride,
-							INT32_MAX),
-			SPA_PARAM_BUFFERS_stride,  SPA_POD_Int(port->stride),
-			SPA_PARAM_BUFFERS_align,   SPA_POD_Int(16));
+			param = spa_pod_builder_add_object(&b,
+				SPA_TYPE_OBJECT_ParamBuffers, id,
+				SPA_PARAM_BUFFERS_buffers, SPA_POD_CHOICE_RANGE_Int(buffers, 1, MAX_BUFFERS),
+				SPA_PARAM_BUFFERS_blocks,  SPA_POD_Int(port->blocks),
+				SPA_PARAM_BUFFERS_size,    SPA_POD_CHOICE_RANGE_Int(
+								size * port->stride,
+								16 * port->stride,
+								INT32_MAX),
+				SPA_PARAM_BUFFERS_stride,  SPA_POD_Int(port->stride),
+				SPA_PARAM_BUFFERS_align,   SPA_POD_Int(16));
+		}
 		break;
 	}
 	case SPA_PARAM_Meta:
 		switch (result.index) {
 		case 0:
+			if (IS_CONTROL_PORT(this, direction, port_id))
+				return -EINVAL;
+
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_ParamMeta, id,
 				SPA_PARAM_META_type, SPA_POD_Id(SPA_META_Header),
@@ -662,22 +734,28 @@ static int port_set_format(void *object,
 		if ((res = spa_format_parse(format, &info.media_type, &info.media_subtype)) < 0)
 			return res;
 
-		if (info.media_type != SPA_MEDIA_TYPE_audio ||
-		    info.media_subtype != SPA_MEDIA_SUBTYPE_raw)
-			return -EINVAL;
+		if (IS_CONTROL_PORT(this, direction, port_id)) {
+			if (info.media_type != SPA_MEDIA_TYPE_application ||
+			    info.media_subtype != SPA_MEDIA_SUBTYPE_control)
+				return -EINVAL;
+		} else {
+			if (info.media_type != SPA_MEDIA_TYPE_audio ||
+			    info.media_subtype != SPA_MEDIA_SUBTYPE_raw)
+				return -EINVAL;
 
-		if (spa_format_audio_raw_parse(format, &info.info.raw) < 0)
-			return -EINVAL;
+			if (spa_format_audio_raw_parse(format, &info.info.raw) < 0)
+				return -EINVAL;
 
-		if (info.info.raw.format != SPA_AUDIO_FORMAT_F32P)
-			return -EINVAL;
+			if (info.info.raw.format != SPA_AUDIO_FORMAT_F32P)
+				return -EINVAL;
 
-		port->stride = sizeof(float);
-		port->blocks = info.info.raw.channels;
+			port->stride = sizeof(float);
+			port->blocks = info.info.raw.channels;
 
-		if (other->have_format) {
-			if ((res = setup_convert(this, direction, &info)) < 0)
-				return res;
+			if (other->have_format) {
+				if ((res = setup_convert(this, direction, &info)) < 0)
+					return res;
+			}
 		}
 		port->format = info;
 		port->have_format = true;
@@ -704,15 +782,20 @@ impl_node_port_set_param(void *object,
 			 uint32_t id, uint32_t flags,
 			 const struct spa_pod *param)
 {
-	spa_return_val_if_fail(object != NULL, -EINVAL);
+	struct impl *this = object;
 
-	spa_return_val_if_fail(CHECK_PORT(object, direction, port_id), -EINVAL);
+	spa_return_val_if_fail(this != NULL, -EINVAL);
 
-	if (id == SPA_PARAM_Format) {
+	spa_return_val_if_fail(CHECK_PORT(this, direction, port_id), -EINVAL);
+
+	switch (id) {
+	case SPA_PARAM_Format:
 		return port_set_format(object, direction, port_id, flags, param);
+	default:
+		break;
 	}
-	else
-		return -ENOENT;
+
+	return -ENOENT;
 }
 
 static int
@@ -730,7 +813,8 @@ impl_node_port_use_buffers(void *object,
 
 	port = GET_PORT(this, direction, port_id);
 
-	spa_return_val_if_fail(port->have_format, -EIO);
+	if (IS_DATA_PORT(this, direction, port_id))
+		spa_return_val_if_fail(port->have_format, -EIO);
 
 	spa_log_debug(this->log, NAME " %p: use buffers %d on port %d", this, n_buffers, port_id);
 
@@ -840,20 +924,89 @@ static int impl_node_port_reuse_buffer(void *object, uint32_t port_id, uint32_t
 	return 0;
 }
 
+static int channelmix_process_control(struct impl *this, struct port *ctrlport,
+				      uint32_t n_dst, void * SPA_RESTRICT dst[n_dst],
+				      uint32_t n_src, const void * SPA_RESTRICT src[n_src],
+				      uint32_t n_samples)
+{
+	struct spa_pod_control *c, *prev = NULL;
+	uint32_t avail_samples = n_samples;
+	uint32_t i;
+	const float **s = (const float **)src;
+	float **d = (float **)dst;
+
+	SPA_POD_SEQUENCE_FOREACH(ctrlport->ctrl, c) {
+		uint32_t chunk;
+
+		if (avail_samples == 0)
+			return 0;
+
+		/* ignore old control offsets */
+		if (c->offset <= ctrlport->ctrl_offset) {
+			prev = c;
+			continue;
+		}
+
+		switch (c->type) {
+		case SPA_CONTROL_Midi:
+		{
+			if (prev)
+				apply_midi(this, &prev->value);
+			break;
+		}
+		case SPA_CONTROL_Properties:
+		{
+			if (prev)
+				apply_props(this, &prev->value);
+			break;
+		}
+		default:
+			continue;
+		}
+
+		chunk = SPA_MIN(avail_samples, c->offset - ctrlport->ctrl_offset);
+
+		spa_log_trace_fp(this->log, NAME " %p: process %d %d", this,
+				c->offset, chunk);
+
+		channelmix_process(&this->mix, n_dst, dst, n_src, src, chunk);
+		for (i = 0; i < n_src; i++)
+			s[i] += chunk;
+		for (i = 0; i < n_dst; i++)
+			d[i] += chunk;
+
+		avail_samples -= chunk;
+		ctrlport->ctrl_offset += chunk;
+
+		prev = c;
+	}
+
+	/* when we get here we run out of control points but still have some
+	 * remaining samples */
+	spa_log_trace_fp(this->log, NAME " %p: remain %d", this, avail_samples);
+	if (avail_samples > 0)
+		channelmix_process(&this->mix, n_dst, dst, n_src, src, avail_samples);
+
+	return 1;
+}
+
 static int impl_node_process(void *object)
 {
 	struct impl *this = object;
-	struct port *outport, *inport;
-	struct spa_io_buffers *outio, *inio;
+	struct port *outport, *inport, *ctrlport;
+	struct spa_io_buffers *outio, *inio, *ctrlio;
 	struct buffer *sbuf, *dbuf;
+	struct spa_pod_sequence *ctrl = NULL;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
 	outport = GET_OUT_PORT(this, 0);
 	inport = GET_IN_PORT(this, 0);
+	ctrlport = GET_CONTROL_PORT(this, 1);
 
 	outio = outport->io;
 	inio = inport->io;
+	ctrlio = ctrlport->io;
 
 	spa_return_val_if_fail(outio != NULL, -EIO);
 	spa_return_val_if_fail(inio != NULL, -EIO);
@@ -864,15 +1017,13 @@ static int impl_node_process(void *object)
 
 	if (SPA_UNLIKELY(outio->status == SPA_STATUS_HAVE_DATA))
 		return SPA_STATUS_HAVE_DATA;
-
-	if (SPA_UNLIKELY(inio->status != SPA_STATUS_HAVE_DATA))
-		return SPA_STATUS_NEED_DATA;
-
 	/* recycle */
 	if (SPA_LIKELY(outio->buffer_id < outport->n_buffers)) {
 		recycle_buffer(this, outio->buffer_id);
 		outio->buffer_id = SPA_ID_INVALID;
 	}
+	if (SPA_UNLIKELY(inio->status != SPA_STATUS_HAVE_DATA))
+		return outio->status = inio->status;
 
 	if (SPA_UNLIKELY(inio->buffer_id >= inport->n_buffers))
 		return inio->status = -EINVAL;
@@ -882,6 +1033,21 @@ static int impl_node_process(void *object)
 
 	sbuf = &inport->buffers[inio->buffer_id];
 
+	if (ctrlio != NULL &&
+	    ctrlio->status == SPA_STATUS_HAVE_DATA &&
+	    ctrlio->buffer_id < ctrlport->n_buffers) {
+		struct buffer *cbuf = &ctrlport->buffers[ctrlio->buffer_id];
+		struct spa_data *d = &cbuf->outbuf->datas[0];
+
+		ctrl = spa_pod_from_data(d->data, d->maxsize, d->chunk->offset, d->chunk->size);
+		if (ctrl && !spa_pod_is_sequence(&ctrl->pod))
+			ctrl = NULL;
+		if (ctrl != ctrlport->ctrl) {
+			ctrlport->ctrl = ctrl;
+			ctrlport->ctrl_offset = 0;
+		}
+	}
+
 	{
 		uint32_t i, n_samples;
 		struct spa_buffer *sb = sbuf->outbuf, *db = dbuf->outbuf;
@@ -891,7 +1057,7 @@ static int impl_node_process(void *object)
 		void *dst_datas[n_dst_datas];
 		bool is_passthrough;
 
-		is_passthrough = this->is_passthrough && this->mix.identity;
+		is_passthrough = this->is_passthrough && this->mix.identity && ctrlport->ctrl == NULL;
 
 		n_samples = sb->datas[0].chunk->size / inport->stride;
 
@@ -903,9 +1069,22 @@ static int impl_node_process(void *object)
 			db->datas[i].chunk->size = n_samples * outport->stride;
 		}
 
-		if (!is_passthrough)
-			channelmix_process(&this->mix, n_dst_datas, dst_datas,
-				    n_src_datas, src_datas, n_samples);
+		spa_log_trace_fp(this->log, NAME " %p: n_src:%d n_dst:%d n_samples:%d p:%d",
+				this, n_src_datas, n_dst_datas, n_samples, is_passthrough);
+
+		if (!is_passthrough) {
+			if (ctrlport->ctrl != NULL) {
+				/* if return value is 1, the sequence has been processed */
+				if (channelmix_process_control(this, ctrlport, n_dst_datas, dst_datas,
+						n_src_datas, src_datas, n_samples) == 1) {
+					ctrlio->status = SPA_STATUS_OK;
+					ctrlport->ctrl = NULL;
+				}
+			} else {
+				channelmix_process(&this->mix, n_dst_datas, dst_datas,
+						n_src_datas, src_datas, n_samples);
+			}
+		}
 	}
 
 	outio->status = SPA_STATUS_HAVE_DATA;
@@ -997,6 +1176,8 @@ impl_init(const struct spa_handle_factory *factory,
 			SPA_NODE_CHANGE_MASK_PARAMS;
 	this->info = SPA_NODE_INFO_INIT();
 	this->info.flags = SPA_NODE_FLAG_RT;
+	this->info.max_input_ports = 2;
+	this->info.max_output_ports = 1;
 	this->params[0] = SPA_PARAM_INFO(SPA_PARAM_PropInfo, SPA_PARAM_INFO_READ);
 	this->params[1] = SPA_PARAM_INFO(SPA_PARAM_Props, SPA_PARAM_INFO_READWRITE);
 	this->info.params = this->params;
@@ -1036,6 +1217,23 @@ impl_init(const struct spa_handle_factory *factory,
 	port->info.n_params = 0;
 	spa_list_init(&port->queue);
 
+	port = GET_CONTROL_PORT(this, 1);
+	port->direction = SPA_DIRECTION_INPUT;
+	port->id = 1;
+	port->info_all = SPA_PORT_CHANGE_MASK_FLAGS |
+			SPA_PORT_CHANGE_MASK_PROPS |
+			SPA_PORT_CHANGE_MASK_PARAMS;
+	port->info = SPA_PORT_INFO_INIT();
+	port->info.flags = SPA_PORT_FLAG_NO_REF |
+		SPA_PORT_FLAG_DYNAMIC_DATA;
+	port->params[0] = SPA_PARAM_INFO(SPA_PARAM_EnumFormat, SPA_PARAM_INFO_READ);
+	port->params[1] = SPA_PARAM_INFO(SPA_PARAM_IO, SPA_PARAM_INFO_READ);
+	port->params[2] = SPA_PARAM_INFO(SPA_PARAM_Format, SPA_PARAM_INFO_WRITE);
+	port->params[3] = SPA_PARAM_INFO(SPA_PARAM_Buffers, 0);
+	port->info.params = port->params;
+	port->info.n_params = 4;
+	spa_list_init(&port->queue);
+
 	return 0;
 }
 
