@@ -49,10 +49,9 @@
 #define DEFAULT_CHANNELS	2
 
 #define MAX_SAMPLES	8192
-#define MAX_BUFFERS	64
+#define MAX_BUFFERS	32
 #define MAX_ALIGN	16
-#define MAX_DATAS	32
-#define MAX_PORTS	128
+#define MAX_DATAS	64
 
 #define PROP_DEFAULT_TRUNCATE	false
 #define PROP_DEFAULT_DITHER	0
@@ -605,7 +604,7 @@ static int port_set_format(void *object,
 			return -EINVAL;
 
 		if (other->have_format) {
-			spa_log_info(this->log, NAME "%p: channels:%d<>%d rate:%d<>%d format:%d<>%d", this,
+			spa_log_debug(this->log, NAME "%p: channels:%d<>%d rate:%d<>%d format:%d<>%d", this,
 				info.info.raw.channels, other->format.info.raw.channels,
 				info.info.raw.rate, other->format.info.raw.rate,
 				info.info.raw.format, other->format.info.raw.format);
@@ -617,8 +616,7 @@ static int port_set_format(void *object,
 
 		if (SPA_AUDIO_FORMAT_IS_PLANAR(info.info.raw.format)) {
 			port->blocks = info.info.raw.channels;
-		}
-		else {
+		} else {
 			port->stride *= info.info.raw.channels;
 			port->blocks = 1;
 		}
@@ -844,7 +842,8 @@ static int impl_node_process(void *object)
 		outio->buffer_id = SPA_ID_INVALID;
 	}
 	if (SPA_UNLIKELY(inio->status != SPA_STATUS_HAVE_DATA))
-		return SPA_STATUS_NEED_DATA;
+		return outio->status = inio->status;
+
 	if (SPA_UNLIKELY(inio->buffer_id >= inport->n_buffers))
 		return inio->status = -EINVAL;
 
