@@ -53,9 +53,9 @@
 
 #define MAX_SAMPLES	8192
 #define MAX_ALIGN	16
-#define MAX_BUFFERS	64
-#define MAX_DATAS	32
-#define MAX_PORTS	128
+#define MAX_BUFFERS	32
+#define MAX_DATAS	64
+#define MAX_PORTS	64
 
 struct buffer {
 	uint32_t id;
@@ -116,7 +116,7 @@ struct impl {
 	unsigned int monitor:1;
 	unsigned int have_profile:1;
 
-	float empty[MAX_SAMPLES*2 + MAX_ALIGN];
+	float empty[MAX_SAMPLES + MAX_ALIGN];
 };
 
 #define CHECK_IN_PORT(this,d,p)		((d) == SPA_DIRECTION_INPUT && (p) < this->port_count)
@@ -167,7 +167,8 @@ static int init_port(struct impl *this, enum spa_direction direction, uint32_t p
 	port->direction = direction;
 	port->id = port_id;
 
-	snprintf(port->position, 16, "%s", rindex(spa_type_audio_channel[position].name, ':')+1);
+	snprintf(port->position, 16, "%s",
+			spa_debug_type_short_name(spa_type_audio_channel[position].name));
 
 	port->info_all = SPA_PORT_CHANGE_MASK_FLAGS |
 			SPA_PORT_CHANGE_MASK_PROPS |
@@ -1006,9 +1007,11 @@ static int impl_node_process(void *object)
 		dbuf->buf->datas[i].data = dst_datas[i];
 		dbuf->buf->datas[i].chunk->offset = 0;
 		dbuf->buf->datas[i].chunk->size = n_samples * outport->stride;
-		spa_log_trace_fp(this->log, NAME " %p %p %d", this, dst_datas[i],
-				n_samples * outport->stride);
 	}
+
+	spa_log_trace_fp(this->log, NAME " %p: n_src:%d n_dst:%d n_samples:%d max:%d p:%d", this,
+			n_src_datas, n_dst_datas, n_samples, maxsize, this->is_passthrough);
+
 	if (!this->is_passthrough)
 		convert_process(&this->conv, dst_datas, src_datas, n_samples);
 
