@@ -133,6 +133,8 @@ struct impl {
 	int frame_count;
 	uint16_t seqnum;
 	uint32_t timestamp;
+	uint8_t tmp_buffer[512];
+	int tmp_buffer_used;
 
 	int min_bitpool;
 	int max_bitpool;
@@ -397,6 +399,8 @@ static int encode_buffer(struct impl *this, const void *data, int size)
 	int processed;
 	ssize_t out_encoded;
 	struct port *port = &this->port;
+	const void *from_data = data;
+	int from_size = size;
 
 	spa_log_trace(this->log, NAME " %p: encode %d used %d, %d %d %d/%d",
 			this, size, this->buffer_used, port->frame_size, this->write_size,
@@ -405,7 +409,18 @@ static int encode_buffer(struct impl *this, const void *data, int size)
 	if (this->frame_count > MAX_FRAME_COUNT)
 		return -ENOSPC;
 
-	processed = sbc_encode(&this->sbc, data, size,
+	if (size < this->codesize - this->tmp_buffer_used) {
+		memcpy(this->tmp_buffer + this->tmp_buffer_used, data, size);
+		this->tmp_buffer_used += size;
+		return size;
+	} else if (this->tmp_buffer_used > 0) {
+		memcpy(this->tmp_buffer + this->tmp_buffer_used, data, this->codesize - this->tmp_buffer_used);
+		from_data = this->tmp_buffer;
+		from_size = this->codesize;
+		this->tmp_buffer_used = this->codesize - this->tmp_buffer_used;
+	}
+
+	processed = sbc_encode(&this->sbc, from_data, from_size,
 			       this->buffer + this->buffer_used,
 			       this->write_size - this->buffer_used,
 			       &out_encoded);
@@ -420,6 +435,11 @@ static int encode_buffer(struct impl *this, const void *data, int size)
 	spa_log_trace(this->log, NAME " %p: processed %d %zd used %d",
 			this, processed, out_encoded, this->buffer_used);
 
+	if (this->tmp_buffer_used) {
+		processed = this->tmp_buffer_used;
+		this->tmp_buffer_used = 0;
+	}
+
 	return processed;
 }
 
@@ -504,6 +524,9 @@ static int set_bitpool(struct impl *this, int bitpool)
 	spa_log_debug(this->log, NAME" %p: set bitpool %d", this, this->sbc.bitpool);
 
 	this->codesize = sbc_get_codesize(&this->sbc);
+	/* make sure there's enough space in this->tmp_buffer */
+	spa_assert(this->codesize <= 512);
+
 	this->frame_length = sbc_get_frame_length(&this->sbc);
 
 	this->read_size = this->transport->read_mtu
@@ -563,7 +586,10 @@ static int flush_data(struct impl *this, uint64_t now_time)
 		if (written > 0 && l1 > 0)
 			written += add_data(this, src, l1);
 		if (written <= 0) {
-			port->need_data = true;
+		        /* only request new data when the current buffer will be fully processed in the next iteration */
+			if (port->ready_offset + (this->frame_count * this->codesize) >= d[0].chunk->size)
+			        port->need_data = true;
+
 			if (written < 0 && written != -ENOSPC) {
 				spa_list_remove(&b->link);
 				b->outstanding = true;
@@ -1167,7 +1193,7 @@ static int port_set_format(struct impl *this, struct port *port,
 	int err;
 
 	if (format == NULL) {
-		spa_log_info(this->log, "clear format");
+		spa_log_debug(this->log, "clear format");
 		clear_buffers(this, port);
 		port->have_format = false;
 	} else {
@@ -1245,7 +1271,7 @@ impl_node_port_use_buffers(void *object,
 	spa_return_val_if_fail(CHECK_PORT(this, direction, port_id), -EINVAL);
 	port = &this->port;
 
-	spa_log_info(this->log, "use buffers %d", n_buffers);
+	spa_log_debug(this->log, "use buffers %d", n_buffers);
 
 	if (!port->have_format)
 		return -EIO;
@@ -1397,6 +1423,8 @@ static int impl_get_interface(struct spa_handle *handle, const char *type, void
 static int impl_clear(struct spa_handle *handle)
 {
 	struct impl *this = (struct impl *) handle;
+	if (this->transport)
+		spa_hook_remove(&this->transport_listener);
 	spa_system_close(this->data_system, this->timerfd);
 	return 0;
 }
