@@ -528,7 +528,7 @@ impl_node_port_enum_params(void *object, int seq,
 static int clear_buffers(struct impl *this, struct port *port)
 {
 	if (port->n_buffers > 0) {
-		spa_log_info(this->log, NAME " %p: clear buffers", this);
+		spa_log_debug(this->log, NAME " %p: clear buffers", this);
 		port->n_buffers = 0;
 		spa_list_init(&port->empty);
 		this->started = false;
@@ -820,8 +820,6 @@ impl_init(const struct spa_handle_factory *factory,
 
 	spa_list_init(&port->empty);
 
-	spa_log_info(this->log, NAME " %p: initialized", this);
-
 	return 0;
 }
 
