@@ -576,12 +576,14 @@ static int port_set_format(void *object,
 	struct port *port = GET_PORT(this, direction, port_id);
 	int res;
 
+	if (port->have_format) {
+		spa_v4l2_stream_off(this);
+		spa_v4l2_clear_buffers(this);
+	}
 	if (format == NULL) {
 		if (!port->have_format)
 			return 0;
 
-		spa_v4l2_stream_off(this);
-		spa_v4l2_clear_buffers(this);
 		port->have_format = false;
 		port->dev.have_format = false;
 		spa_v4l2_close(&port->dev);
@@ -601,48 +603,28 @@ static int port_set_format(void *object,
 				spa_log_error(this->log, "can't parse video raw");
 				return -EINVAL;
 			}
-
-			if (port->have_format && info.media_type == port->current_format.media_type &&
-			    info.media_subtype == port->current_format.media_subtype &&
-			    info.info.raw.format == port->current_format.info.raw.format &&
-			    info.info.raw.size.width == port->current_format.info.raw.size.width &&
-			    info.info.raw.size.height == port->current_format.info.raw.size.height)
-				return 0;
 			break;
 		case SPA_MEDIA_SUBTYPE_mjpg:
 			if (spa_format_video_mjpg_parse(format, &info.info.mjpg) < 0)
 				return -EINVAL;
-
-			if (port->have_format && info.media_type == port->current_format.media_type &&
-			    info.media_subtype == port->current_format.media_subtype &&
-			    info.info.mjpg.size.width == port->current_format.info.mjpg.size.width &&
-			    info.info.mjpg.size.height == port->current_format.info.mjpg.size.height)
-				return 0;
 			break;
 		case SPA_MEDIA_SUBTYPE_h264:
 			if (spa_format_video_h264_parse(format, &info.info.h264) < 0)
 				return -EINVAL;
-
-			if (port->have_format && info.media_type == port->current_format.media_type &&
-			    info.media_subtype == port->current_format.media_subtype &&
-			    info.info.h264.size.width == port->current_format.info.h264.size.width &&
-			    info.info.h264.size.height == port->current_format.info.h264.size.height)
-				return 0;
 			break;
 		default:
 			return -EINVAL;
 		}
 	}
 
-	if (port->have_format && !(flags & SPA_NODE_PARAM_FLAG_TEST_ONLY)) {
-		spa_v4l2_use_buffers(this, NULL, 0);
+	if (port->have_format && !SPA_FLAG_IS_SET(flags, SPA_NODE_PARAM_FLAG_TEST_ONLY)) {
 		port->have_format = false;
 	}
 
-	if (spa_v4l2_set_format(this, &info, flags & SPA_NODE_PARAM_FLAG_TEST_ONLY) < 0)
+	if (spa_v4l2_set_format(this, &info, flags) < 0)
 		return -EINVAL;
 
-	if (!(flags & SPA_NODE_PARAM_FLAG_TEST_ONLY)) {
+	if (!SPA_FLAG_IS_SET(flags, SPA_NODE_PARAM_FLAG_TEST_ONLY)) {
 		port->current_format = info;
 		port->have_format = true;
 	}
