@@ -28,14 +28,17 @@
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <sys/socket.h>
 
-#include <spa/debug/pod.h>
 #include <spa/utils/result.h>
 #include <spa/pod/builder.h>
 
 #include <pipewire/pipewire.h>
 
+#define spa_debug pw_log_debug
+#include <spa/debug/pod.h>
+
 #include "connection.h"
 
 #define MAX_BUFFER_SIZE (1024 * 32)
@@ -98,7 +101,7 @@ int pw_protocol_native_connection_get_fd(struct pw_protocol_native_connection *c
  *
  * \param conn the connection
  * \param fd the fd to add
- * \return the index of the fd or SPA_IDX_INVALID when an error occured
+ * \return the index of the fd or SPA_IDX_INVALID when an error occurred
  *
  * \memberof pw_protocol_native_connection
  */
@@ -122,7 +125,7 @@ uint32_t pw_protocol_native_connection_add_fd(struct pw_protocol_native_connecti
 		return SPA_IDX_INVALID;
 	}
 
-	buf->msg.fds[index] = fd;
+	buf->msg.fds[index] = fcntl(fd, F_DUPFD_CLOEXEC, 0);
 	buf->msg.n_fds++;
 	pw_log_debug("connection %p: add fd %d at index %d", conn, fd, index);
 
@@ -208,8 +211,13 @@ recv_error:
 	return -errno;
 }
 
-static void clear_buffer(struct buffer *buf)
+static void clear_buffer(struct buffer *buf, bool fds)
 {
+	uint32_t i;
+	if (fds) {
+		for (i = 0; i < buf->n_fds; i++)
+			close(buf->fds[i]);
+	}
 	buf->n_fds = 0;
 	buf->buffer_size = 0;
 	buf->offset = 0;
@@ -237,7 +245,7 @@ struct pw_protocol_native_connection *pw_protocol_native_connection_new(struct p
 
 	this = &impl->this;
 
-	pw_log_debug("connection %p: new", this);
+	pw_log_debug("connection %p: new fd:%d", this, fd);
 
 	this->fd = fd;
 	spa_hook_list_init(&this->listener_list);
@@ -264,6 +272,7 @@ no_mem:
 
 int pw_protocol_native_connection_set_fd(struct pw_protocol_native_connection *conn, int fd)
 {
+	pw_log_debug("connection %p: fd:%d", conn, fd);
 	conn->fd = fd;
 	return 0;
 }
@@ -282,6 +291,8 @@ void pw_protocol_native_connection_destroy(struct pw_protocol_native_connection
 
 	spa_hook_list_call(&conn->listener_list, struct pw_protocol_native_connection_events, destroy, 0);
 
+	clear_buffer(&impl->out, true);
+	clear_buffer(&impl->in, true);
 	free(impl->out.buffer_data);
 	free(impl->in.buffer_data);
 	free(impl);
@@ -341,7 +352,7 @@ static int prepare_packet(struct pw_protocol_native_connection *conn, struct buf
 	buf->fds_offset += buf->msg.n_fds;
 
 	if (buf->offset >= buf->buffer_size)
-		clear_buffer(buf);
+		clear_buffer(buf, false);
 
 	return 0;
 }
@@ -349,8 +360,8 @@ static int prepare_packet(struct pw_protocol_native_connection *conn, struct buf
 /** Move to the next packet in the connection
  *
  * \param conn the connection
- * \param opcode addres of result opcode
- * \param dest_id addres of result destination id
+ * \param opcode address of result opcode
+ * \param dest_id address of result destination id
  * \param dt pointer to packet data
  * \param sz size of packet data
  * \return true on success
@@ -466,7 +477,7 @@ pw_protocol_native_connection_end(struct pw_protocol_native_connection *conn,
 		buf->n_fds = buf->msg.n_fds;
 
 	if (debug_messages) {
-		fprintf(stderr, ">>>>>>>>> out: id:%d op:%d size:%d seq:%d\n",
+		pw_log_debug(">>>>>>>>> out: id:%d op:%d size:%d seq:%d",
 				buf->msg.id, buf->msg.opcode, size, buf->msg.seq);
 	        spa_debug_pod(0, NULL, SPA_MEMBER(p, impl->hdr_size, struct spa_pod));
 	}
@@ -498,7 +509,7 @@ int pw_protocol_native_connection_flush(struct pw_protocol_native_connection *co
 	struct cmsghdr *cmsg;
 	char cmsgbuf[CMSG_SPACE(MAX_FDS_MSG * sizeof(int))];
 	int res = 0, *fds;
-	uint32_t fds_len, n_fds, outfds;
+	uint32_t fds_len, to_close, n_fds, outfds, i;
 	struct buffer *buf;
 	void *data;
 	size_t size;
@@ -508,6 +519,7 @@ int pw_protocol_native_connection_flush(struct pw_protocol_native_connection *co
 	size = buf->buffer_size;
 	fds = buf->fds;
 	n_fds = buf->n_fds;
+	to_close = 0;
 
 	while (size > 0) {
 		if (n_fds > MAX_FDS_MSG) {
@@ -558,6 +570,7 @@ int pw_protocol_native_connection_flush(struct pw_protocol_native_connection *co
 		data = SPA_MEMBER(data, sent, void);
 		n_fds -= outfds;
 		fds += outfds;
+		to_close += outfds;
 	}
 
 	res = 0;
@@ -566,6 +579,8 @@ exit:
 	if (size > 0)
 		memmove(buf->buffer_data, data, size);
 	buf->buffer_size = size;
+	for (i = 0; i < to_close; i++)
+		close(buf->fds[i]);
 	if (n_fds > 0)
 		memmove(buf->fds, fds, n_fds * sizeof(int));
 	buf->n_fds = n_fds;
@@ -585,8 +600,8 @@ int pw_protocol_native_connection_clear(struct pw_protocol_native_connection *co
 {
 	struct impl *impl = SPA_CONTAINER_OF(conn, struct impl, this);
 
-	clear_buffer(&impl->out);
-	clear_buffer(&impl->in);
+	clear_buffer(&impl->out, true);
+	clear_buffer(&impl->in, true);
 
 	return 0;
 }
