@@ -967,7 +967,11 @@ filter_new(struct pw_context *context, const char *name,
 			str = pw_properties_get(extra, PW_KEY_APP_PROCESS_BINARY);
 		if (str == NULL)
 			str = name;
-		pw_properties_set(props, PW_KEY_NODE_NAME, name);
+		pw_properties_set(props, PW_KEY_NODE_NAME, str);
+	}
+	if (pw_properties_get(props, PW_KEY_NODE_LATENCY) == NULL) {
+		if ((str = getenv("PIPEWIRE_LATENCY")) != NULL)
+			pw_properties_set(props, PW_KEY_NODE_LATENCY, str);
 	}
 
 	spa_hook_list_init(&impl->hooks);
