@@ -856,16 +856,21 @@ static void output_node_result(void *data, int seq, int res, uint32_t type, cons
 	node_result(impl, port, seq, res, type, result);
 }
 
-static void node_active_changed(void *data, bool active)
+static void check_prepare(struct pw_impl_link *this)
 {
-	struct impl *impl = data;
-	struct pw_impl_link *this = &impl->this;
+	struct impl *impl = SPA_CONTAINER_OF(this, struct impl, this);
 	pw_log_debug(NAME" %p: input active:%d output active:%d", impl,
 			impl->inode->active, impl->onode->active);
 	if (impl->inode->active && impl->onode->active)
 		pw_impl_link_prepare(this);
 }
 
+static void node_active_changed(void *data, bool active)
+{
+	struct impl *impl = data;
+	check_prepare(&impl->this);
+}
+
 static const struct pw_impl_node_events input_node_events = {
 	PW_VERSION_IMPL_NODE_EVENTS,
 	.result = input_node_result,
@@ -1177,17 +1182,14 @@ int pw_impl_link_register(struct pw_impl_link *link,
 		PW_KEY_MODULE_ID,
 		PW_KEY_FACTORY_ID,
 		PW_KEY_CLIENT_ID,
+		PW_KEY_LINK_OUTPUT_PORT,
+		PW_KEY_LINK_INPUT_PORT,
 		NULL
 	};
 
 	if (link->registered)
 		goto error_existed;
 
-	if (properties == NULL)
-		properties = pw_properties_new(NULL, NULL);
-	if (properties == NULL)
-		return -errno;
-
 	output_node = link->output->node;
 	input_node = link->input->node;
 
@@ -1196,11 +1198,6 @@ int pw_impl_link_register(struct pw_impl_link *link,
 	link->info.input_node_id = input_node->global->id;
 	link->info.input_port_id = link->input->global->id;
 
-	pw_properties_update_keys(properties, &link->properties->dict, keys);
-
-	pw_properties_setf(properties, PW_KEY_LINK_OUTPUT_PORT, "%d", link->info.output_port_id);
-	pw_properties_setf(properties, PW_KEY_LINK_INPUT_PORT, "%d", link->info.input_port_id);
-
 	link->global = pw_global_new(context,
 				     PW_TYPE_INTERFACE_Link,
 				     PW_VERSION_LINK,
@@ -1215,15 +1212,18 @@ int pw_impl_link_register(struct pw_impl_link *link,
 
 	link->info.id = link->global->id;
 	pw_properties_setf(link->properties, PW_KEY_OBJECT_ID, "%d", link->info.id);
+	pw_properties_setf(link->properties, PW_KEY_LINK_OUTPUT_PORT, "%d", link->info.output_port_id);
+	pw_properties_setf(link->properties, PW_KEY_LINK_INPUT_PORT, "%d", link->info.input_port_id);
 	link->info.props = &link->properties->dict;
 
+	pw_global_update_keys(link->global, link->info.props, keys);
+
 	pw_impl_link_emit_initialized(link);
 
 	pw_global_add_listener(link->global, &link->global_listener, &global_events, link);
 	pw_global_register(link->global);
 
-	if (input_node->active && output_node->active)
-		pw_impl_link_prepare(link);
+	check_prepare(link);
 
 	return 0;
 
