@@ -246,7 +246,7 @@ static void draw_node(struct global *g)
 	struct global *p;
 	const char *prop_node_id;
 	spa_list_for_each(p, &g->data->globals, link) {
-		if (p == NULL || p->info == NULL)
+		if (p->info == NULL)
 			continue;
 		if (p->type != INTERFACE_Port)
 			continue;
@@ -407,7 +407,7 @@ static bool is_node_id_link_referenced(uint32_t id, struct spa_list *globals)
         struct global *g;
         struct pw_link_info *info;
         spa_list_for_each(g, globals, link) {
-                if (g == NULL || g->info == NULL)
+                if (g->info == NULL)
                         continue;
                 if (g->type != INTERFACE_Link)
                         continue;
@@ -424,7 +424,7 @@ static bool is_module_id_factory_referenced(uint32_t id, struct spa_list *global
         struct pw_factory_info *info;
         const char *module_id_str;
         spa_list_for_each(g, globals, link) {
-                if (g == NULL || g->info == NULL)
+                if (g->info == NULL)
                         continue;
                 if (g->type != INTERFACE_Factory)
                         continue;
@@ -461,7 +461,7 @@ static int draw_graph(struct data *d, const char *path)
 	/* iterate the globals */
 	spa_list_for_each(g, &d->globals, link) {
 		/* skip null and non-info globals */
-		if (g == NULL || g->info == NULL)
+		if (g->info == NULL)
 			continue;
 
 		/* always skip ports since they are drawn by the nodes */
@@ -496,7 +496,7 @@ static int draw_graph(struct data *d, const char *path)
 	/* open the file */
 	fp = fopen(path, "w");
 	if (fp == NULL) {
-		printf("open error: could not open %s for writting\n", path);
+		printf("open error: could not open %s for writing\n", path);
 		return -1;
 	}
 
@@ -582,23 +582,24 @@ static const struct pw_module_events module_events = {
 	.info = module_event_info
 };
 
+static void removed_proxy(void *user_data)
+{
+	struct global *g = user_data;
+	pw_proxy_destroy(g->proxy);
+}
+
 static void destroy_proxy(void *user_data)
 {
-        struct global *g = user_data;
-
-        if (g->props) {
-                pw_properties_free(g->props);
-                g->props = NULL;
-        }
-
-        if (g->info) {
-                g->info_destroy(g->info);
-                g->info = NULL;
-        }
+	struct global *g = user_data;
+	if (g->props)
+		pw_properties_free(g->props);
+	if (g->info)
+		g->info_destroy(g->info);
 }
 
 static const struct pw_proxy_events proxy_events = {
 	PW_VERSION_PROXY_EVENTS,
+	.removed = removed_proxy,
 	.destroy = destroy_proxy,
 };
 
@@ -746,15 +747,15 @@ static void do_quit(void *data, int signal_number)
 static void show_help(const char *name)
 {
         fprintf(stdout, "%s [options]\n"
-             "  -h, --help                            Show this help\n"
-             "  -v, --version                         Show version\n"
-             "  -a, --all                             Show all object types\n"
-             "  -s, --smart                           Show linked objects only\n"
-             "  -d, --detail                          Show all object properties\n"
-             "  -r, --remote                          Remote daemon name\n"
-             "  -o, --output                          Output file (Default %s)\n",
-             name,
-             DEFAULT_DOT_PATH);
+		"  -h, --help                            Show this help\n"
+		"      --version                         Show version\n"
+		"  -a, --all                             Show all object types\n"
+		"  -s, --smart                           Show linked objects only\n"
+		"  -d, --detail                          Show all object properties\n"
+		"  -r, --remote                          Remote daemon name\n"
+		"  -o, --output                          Output file (Default %s)\n",
+		name,
+		DEFAULT_DOT_PATH);
 }
 
 int main(int argc, char *argv[])
@@ -764,25 +765,25 @@ int main(int argc, char *argv[])
 	const char *opt_remote = NULL;
 	const char *dot_path = DEFAULT_DOT_PATH;
 	static const struct option long_options[] = {
-		{"help",	0, NULL, 'h'},
-		{"version",	0, NULL, 'v'},
-		{"all",		0, NULL, 'a'},
-		{"smart",	0, NULL, 's'},
-		{"detail",	0, NULL, 'd'},
-		{"remote",	1, NULL, 'r'},
-		{"output",	1, NULL, 'o'},
-		{NULL,		0, NULL, 0}
+		{ "help",	no_argument,		NULL, 'h' },
+		{ "version",	no_argument,		NULL, 'V' },
+		{ "all",	no_argument,		NULL, 'a' },
+		{ "smart",	no_argument,		NULL, 's' },
+		{ "detail",	no_argument,		NULL, 'd' },
+		{ "remote",	required_argument,	NULL, 'r' },
+		{ "output",	required_argument,	NULL, 'o' },
+		{ NULL, 0, NULL, 0}
 	};
 	int c;
 
 	pw_init(&argc, &argv);
 
-	while ((c = getopt_long(argc, argv, "hvasdr:o:", long_options, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "hVasdr:o:", long_options, NULL)) != -1) {
 		switch (c) {
 		case 'h' :
 			show_help(argv[0]);
 			return 0;
-		case 'v' :
+		case 'V' :
 			fprintf(stdout, "%s\n"
 				"Compiled with libpipewire %s\n"
 				"Linked with libpipewire %s\n",
@@ -811,29 +812,36 @@ int main(int argc, char *argv[])
 			fprintf(stdout, "set output file %s\n", dot_path);
 			break;
 		default:
+			show_help(argv[0]);
 			return -1;
 		}
 	}
 
 	data.loop = pw_main_loop_new(NULL);
-	if (data.loop == NULL)
+	if (data.loop == NULL) {
+		fprintf(stderr, "can't create main loop: %m\n");
 		return -1;
+	}
 
 	l = pw_main_loop_get_loop(data.loop);
 	pw_loop_add_signal(l, SIGINT, do_quit, &data);
 	pw_loop_add_signal(l, SIGTERM, do_quit, &data);
 
 	data.context = pw_context_new(l, NULL, 0);
-	if (data.context == NULL)
+	if (data.context == NULL) {
+		fprintf(stderr, "can't create context: %m\n");
 		return -1;
+	}
 
 	data.core = pw_context_connect(data.context,
 			pw_properties_new(
 				PW_KEY_REMOTE_NAME, opt_remote,
 				NULL),
 			0);
-	if (data.core == NULL)
+	if (data.core == NULL) {
+		fprintf(stderr, "can't connect: %m\n");
 		return -1;
+	}
 
 	data.dot_str = dot_str_new();
 	if (data.dot_str == NULL)
@@ -855,8 +863,10 @@ int main(int argc, char *argv[])
 	draw_graph(&data, dot_path);
 
 	dot_str_clear(&data.dot_str);
+	pw_proxy_destroy((struct pw_proxy*)data.registry);
 	pw_context_destroy(data.context);
 	pw_main_loop_destroy(data.loop);
+	pw_deinit();
 
 	return 0;
 }
